import QtQuick 2.0
import Ubuntu.Components 0.1
import hello 1.0

/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "mainView"

    // Note! applicationName needs to match the "name" field of the click manifest
    applicationName: "com.ubuntu.developer.ralsina.hello"

    /*
     This property enables the application to change orientation
     when the device is rotated. The default is false.
    */
    //automaticOrientation: true

    width: units.gu(100)
    height: units.gu(75)
    ChatClient {
        id: chatClient
        Component.onCompleted: {
            error.connect(messageList.handle_error)
            registered.connect(nick.registered)
        }
    }

    PushClient {
        id: pushClient
        Component.onCompleted: {
            new_notifications.connect(messageList.handle_notifications)
            error.connect(messageList.handle_error)
        }
        appid: "com.ubuntu.developer.ralsina.hello_hello"

    }

    TextField {
        id: nick
        focus: true
        placeholderText: "Your nickname"
        anchors.left: parent.left
        anchors.right: parent.right
        function registered() {
            nick.readOnly = true
            nick.text = "Your nick is " + chatClient.nick
            message.focus = true
        }
        onAccepted: {
            chatClient.registerNick(nick.text, pushClient.token)
        }
    }
    TextField {
        id: message
        anchors.right: annoyingSwitch.left
        anchors.left: parent.left
        anchors.top: nick.bottom
        anchors.topMargin: units.gu(1)
        anchors.rightMargin: units.gu(1)
        placeholderText: "Your message"
        onAccepted: {
            console.log("sending " + text)
            messagesModel.append({
                "direction" :  "◀ ",
                "nick" :  "",
                "message" : text
            })
            chatClient.sendMessage(message.text, annoyingSwitch.checked)
            message.text = ""
        }
    }
    Switch {
        id: annoyingSwitch
        anchors.right: parent.right
        anchors.top: nick.bottom
        anchors.topMargin: units.gu(1)
    }

    ListModel {
        id: messagesModel
        ListElement {
            nick: ""
            direction: "*** "
            message: "Register by typing your nick and pressing enter."
        }
        ListElement {
            nick: ""
            direction: "*** "
            message: "Send messages in the form \"destination: hello\""
        }
        ListElement {
            nick: ""
            direction: "*** "
            message: "The switch on the right makes the message more annoying for the recipient."
        }
    }

    UbuntuShape {
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.top: message.bottom
        anchors.topMargin: units.gu(1)
        ListView {
            id: messageList
            model: messagesModel
            anchors.fill: parent
            delegate: Text {
                text: direction + nick + ": " + message
            }
            function handle_error(error) {
                messagesModel.append({
                    "direction" :  "*** ",
                    "nick" :  "ERROR",
                    "message" : error
                })
            }

            function handle_notifications(list) {
                list.forEach(function(notification) {
                    var item = {}
                    item["nick"] = notification.split(": ")[0]
                    item["message"] = notification.split(": ")[1]
                    item["direction"] =  "▶ "
                    messagesModel.append(item)
                })
            }
        }
    }
}
